const yargs = require("yargs/yargs");
const cli = yargs(process.argv.slice(2));
const pkg = require("../package.json");
const collectComplexity = require("../lib/collectComplexity");

module.exports = function command() {
  // 代码复杂度
  cli
    .demandCommand(
      1,
      "A command is required. Pass --help to see all available commands and options."
    )
    .alias("cc", "codeComplexity")
    .command({
      command: "codeComplexity",
      aliases: "cc",
      describe: "获取代码复杂度详情",
      handler: async (args) => {
        console.log("正在扫描代码,请稍后……");
        const type = args.type || args.t || "bad";
        const start = new Date();
        const { complexitys, result } = await collectComplexity();
        const { fileCount, funcCount, otherCount } = complexitys;
        const badResult = result.filter((item) => item.complexity > 70);
        const badCount = badResult.length;
        console.log(
          `检测完成,耗时 ${
            Date.now() - start
          } ms,共检测 ${fileCount} 个文件, ${funcCount} 个函数,其他类型（除函数外,可能是方法「classMethod 或 objectMethod」） ${otherCount} 个,其中可优化的 ${badCount} 个`
        );
        switch (type) {
          case "bad": {
            if (!badCount) console.log("你太棒啦！代码没问题哦～");
            else {
              console.log("复杂度统计详情：");
              console.table(badResult);
            }
            return;
          }
          case "medium": {
            const mediumResult = result.filter(
              (item) => item.advice === "建议"
            );
            if (!mediumResult.length) console.log("你太棒啦！代码没问题哦～");
            else {
              console.log("复杂度统计详情：");
              console.table(mediumResult);
            }
            return;
          }
          case "good": {
            console.table(result.filter((item) => item.complexity < 60));
            return;
          }
          default:
            console.table(result);
            return;
        }
      },
    });

  // 代码详情
  cli
    .demandCommand(
      1,
      "A command is required. Pass --help to see all available commands and options."
    )
    .alias("cd", "codeDesc")
    .command({
      command: "codeDesc",
      aliases: "cd",
      describe: "获取某文件代码详情",
      handler: async (args) => {
        const path = args.path || args.p;
        if (!path) {
          console.log("请补充指令:如,[-path || --p]='/test/index.js'");
          return;
        }
        console.log("正在扫描代码,请稍后……");
        const { result, fileDetail } = await collectComplexity();
        if (fileDetail.length) {
          console.log("复杂度详情：");
          console.table(result.filter((item) => item.filePath === path));
          console.log("文件扫描详情：");
          console.table(fileDetail.filter((item) => item.filePath === path));
        } else console.log("你太棒啦！代码没问题哦～");
      },
    });

  // 获取当前包的版本
  cli.command({
    command: "version",
    aliases: "v",
    describe: "获取当前包的版本号",
    handler: () => {
      console.log(pkg.version);
    },
  });

  cli.help().argv;
};
